from datetime import datetime, timedelta
from .KowanasTime import KowanasTime

class Scheduler:
    TERM_DAY = 'day'
    TERM_HOUR = 'hour'
    TERM_MIN = 'min'
    TERM_SECOND = 'second'

    def __init__(self, every=None, term=None, begin=None):
        self.schedule = [] 
        self.every = every
        self.term = term
        self.begin = begin

    def setNext(self):
        if len(self.schedule) <= 0:
            self.setBeginTime()

    def setBeginTime(self, plus = 0):
        currentTime = KowanasTime.getKST()
        if self.term == Scheduler.TERM_DAY: 
            beginTime = datetime(year=currentTime.year, month=currentTime.month, day=currentTime.day, hour=0, minute=0, second=0) + timedelta(days=plus)
        elif self.term == Scheduler.TERM_HOUR:
            beginTime = datetime(year=currentTime.year, month=currentTime.month, day=currentTime.day, hour=currentTime.hour, minute=0, second=0) + timedelta(hours=plus)
        elif self.term == Scheduler.TERM_MIN:
            beginTime = datetime(year=currentTime.year, month=currentTime.month, day=currentTime.day, hour=currentTime.hour, minute=currentTime.minute, second=0) + timedelta(minutes=plus)
        elif self.term == Scheduler.TERM_SECOND:
            beginTime = datetime(year=currentTime.year, month=currentTime.month, day=currentTime.day, hour=currentTime.hour, minute=currentTime.minute, second=currentTime.second) + timedelta(seconds=plus)
        self.schedule = [beginTime + every for every in self.every]
        removeCount = 0
        for schedule in self.schedule:
            if schedule < currentTime: removeCount+=1
        self.schedule = self.schedule[removeCount:]
        if len(self.schedule) <= 0: self.setBeginTime(plus=1)

    def getNext(self):
        self.setNext()
        nextTime = self.schedule[0]
        if self.begin != None:
            begin = self.begin
            self.begin = None
            return begin
        return nextTime

    def clearNext(self):
        self.schedule.remove(self.schedule[0])

    @classmethod
    def everysec(self, s):
        return [timedelta(seconds=x) for x in range(0, 60, s)]

    @classmethod
    def everymin(self, m):
        return [timedelta(minutes=x) for x in range(0, 60, m)]


class DBField:
    AutoIncrement = 'AUTO_INCREMENT'
    def __init__(self, key, dtype, length, autoIncrement = ''):
        self.key = key
        self.type = dtype
        self.length = length
        self.autoIncrement = autoIncrement


import time
import threading
from datetime import datetime
from datetime import timedelta

from abc import ABCMeta
from .Log import Log
from .Task import Task
from .KowanasTime import KowanasTime

class Service(metaclass=ABCMeta):
    _name = ''
    _tasks = {}
    def __init__(self, name):
        self._name = name
        self.__log = Log()

    def _addTask(self, task):
        self._tasks[task.name] = task

    def run(self):
        threading.Thread(target=self.__run).start()

    def __run(self):
        while True:
            nextSchedule = None
            for task in list(self._tasks.values()):
                expectSchedule = task.schedule.getNext()
                if expectSchedule < KowanasTime.getKST():
                    task.schedule.clearNext()
                    task.run()
                    expectSchedule = task.schedule.getNext()
                if nextSchedule == None or expectSchedule < nextSchedule:
                    nextSchedule = expectSchedule
            self._waitUntil(nextSchedule)        

    def _handle(self):
        pass

    def _wait(self, duration):
        time.sleep(duration)

    def _waitUntil(self, until):
#        self.__log.d('current time '+KowanasTime.getKST().strftime('%Y-%m-%d %H:%M:%S'))
#        self.__log.d('wait until '+until.strftime('%Y-%m-%d %H:%M:%S'))
        now = KowanasTime.getKST()
        if now < until:
            remained = until - now
            self.__log.d('waiting for '+str(remained.total_seconds())+' seconds')
            time.sleep(remained.total_seconds())


import pymysql
from .Config import Config

class DBConnection:
    db = None
    dbname = ''
    cursor = None
    def __init__(self, dbname, config):
        self.dbname = dbname
        self.__config = config

    def connect(self):
        config = self.__config
        self.db = pymysql.connect(user=config.get('DB_USER'), passwd=config.get('DB_PASSWORD'), host=config.get('DB_HOST'), db=self.dbname, charset='utf8')
        self.cursor = self.db.cursor(pymysql.cursors.DictCursor)

    def disconnect(self):
        self.db.close()

    def commit(self):
        self.db.commit()
